@@ -9,13 +9,13 @@ use lru::LruCache;
 use serde_json::Value;
 use uuid::Uuid;
 
-use crate::addressing::Address;
+use crate::addressing::{Address, Addressable};
 use crate::database::constants::{
     HIER_ADDR, HIER_HAS_ATTR, HIER_INVARIANT, IS_OF_TYPE_ATTR, LABEL_ATTR, TYPE_ADDR, TYPE_HAS_ATTR,
 };
 use crate::database::entry::{Entry, EntryValue};
 use crate::database::lang::{EntryQuery, Query, QueryComponent, QueryPart};
-use crate::database::{bulk_retrieve_objects, insert_entry, query, DbPool};
+use crate::database::{insert_entry, query, DbPool};
 
 #[derive(Debug, Clone, Eq, PartialEq, Hash)]
 pub struct UNode(String);
@@ -37,14 +37,14 @@ impl UNode {
 }
 
 #[derive(Debug, Clone, PartialEq)]
-pub struct UPath(pub Vec<UNode>);
+pub struct UHierPath(pub Vec<UNode>);
 
-impl std::str::FromStr for UPath {
+impl std::str::FromStr for UHierPath {
     type Err = anyhow::Error;
 
     fn from_str(string: &str) -> Result<Self, Self::Err> {
         if string.is_empty() {
-            Ok(UPath(vec![]))
+            Ok(UHierPath(vec![]))
         } else {
             let result: Result<Vec<UNode>> = string
                 .trim_end_matches('/')
@@ -52,7 +52,7 @@ impl std::str::FromStr for UPath {
                 .map(|part| UNode::new(String::from(part)))
                 .collect();
 
-            Ok(UPath(result?))
+            Ok(UHierPath(result?))
         }
     }
 }
@@ -63,7 +63,7 @@ impl std::fmt::Display for UNode {
     }
 }
 
-impl std::fmt::Display for UPath {
+impl std::fmt::Display for UHierPath {
     fn fmt(&self, f: &mut std::fmt::Formatter<'_>) -> std::fmt::Result {
         write!(
             f,
@@ -77,16 +77,16 @@ impl std::fmt::Display for UPath {
     }
 }
 
-trait EntryList {
-    fn extract_addresses(&self) -> Vec<Address>;
+trait PointerEntries {
+    fn extract_pointers(&self) -> Vec<(Address, Address)>;
 }
 
-impl EntryList for Vec<Entry> {
-    fn extract_addresses(&self) -> Vec<Address> {
+impl PointerEntries for Vec<Entry> {
+    fn extract_pointers(&self) -> Vec<(Address, Address)> {
         self.iter()
             .filter_map(|e| {
                 if let EntryValue::Address(address) = &e.value {
-                    Some(address.clone())
+                    Some((e.address().unwrap(), address.clone()))
                 } else {
                     None
                 }
@@ -95,7 +95,7 @@ impl EntryList for Vec<Entry> {
     }
 }
 
-pub fn list_orphans<C: Connection<Backend = Sqlite>>(connection: &C) -> Result<Vec<Address>> {
+pub fn list_roots<C: Connection<Backend = Sqlite>>(connection: &C) -> Result<Vec<Address>> {
     let all_directories: Vec<Entry> = query(
         connection,
         Query::SingleQuery(QueryPart::Matches(EntryQuery {
@@ -105,6 +105,7 @@ pub fn list_orphans<C: Connection<Backend = Sqlite>>(connection: &C) -> Result<V
         })),
     )?;
 
+    // TODO: this is horrible
     let directories_with_parents: Vec<Address> = query(
         connection,
         Query::SingleQuery(QueryPart::Matches(EntryQuery {
@@ -113,7 +114,10 @@ pub fn list_orphans<C: Connection<Backend = Sqlite>>(connection: &C) -> Result<V
             value: QueryComponent::Any,
         })),
     )?
-    .extract_addresses();
+    .extract_pointers()
+    .into_iter()
+    .map(|(_, val)| val)
+    .collect();
 
     Ok(all_directories
         .into_iter()
@@ -122,29 +126,6 @@ pub fn list_orphans<C: Connection<Backend = Sqlite>>(connection: &C) -> Result<V
         .collect())
 }
 
-pub async fn list_node<C: Connection<Backend = Sqlite>>(
-    connection: &C,
-    path: &UPath,
-) -> Result<Vec<Entry>> {
-    let resolved_path: Vec<Address> = resolve_path(connection, path, false)?;
-    let last = resolved_path.last().unwrap();
-
-    let entry_addresses = query(
-        connection,
-        Query::SingleQuery(QueryPart::Matches(EntryQuery {
-            entity: QueryComponent::Exact(last.clone()),
-            attribute: QueryComponent::Exact(HIER_HAS_ATTR.to_string()),
-            value: QueryComponent::Any,
-        })),
-    )?
-    .extract_addresses();
-
-    Ok(bulk_retrieve_objects(connection, entry_addresses)?
-        .into_iter()
-        // .filter(|e| [DIR_KEY, FILENAME_KEY, FILE_IDENTITY_KEY].contains(&e.attribute.as_str()))
-        .collect::<Vec<Entry>>())
-}
-
 lazy_static! {
     static ref FETCH_CREATE_LOCK: Mutex<()> = Mutex::new(());
 }
@@ -187,8 +168,11 @@ pub fn fetch_or_create_dir<C: Connection<Backend = Sqlite>>(
                 value: QueryComponent::Any,
             })),
         )?
-        .extract_addresses(),
-        None => list_orphans(connection)?,
+        .extract_pointers()
+        .into_iter()
+        .map(|(_, val)| val)
+        .collect(),
+        None => list_roots(connection)?,
     };
 
     let valid_directories: Vec<Address> = matching_directories
@@ -237,7 +221,7 @@ pub fn fetch_or_create_dir<C: Connection<Backend = Sqlite>>(
 
 pub fn resolve_path<C: Connection<Backend = Sqlite>>(
     connection: &C,
-    path: &UPath,
+    path: &UHierPath,
     create: bool,
 ) -> Result<Vec<Address>> {
     let mut result: Vec<Address> = vec![];
@@ -261,7 +245,7 @@ pub type ResolveCache = LruCache<(Option<Address>, UNode), Address>;
 
 pub fn resolve_path_cached<C: Connection<Backend = Sqlite>>(
     connection: &C,
-    path: &UPath,
+    path: &UHierPath,
     create: bool,
     cache: &Arc<Mutex<ResolveCache>>,
 ) -> Result<Vec<Address>> {
@@ -315,7 +299,7 @@ mod tests {
 
     #[test]
     fn test_path_codec() {
-        let path = UPath(vec![
+        let path = UHierPath(vec![
             UNode("top".to_string()),
             UNode("foo".to_string()),
             UNode("bar".to_string()),
@@ -325,7 +309,7 @@ mod tests {
         let str_path = path.to_string();
         assert!(!str_path.is_empty());
 
-        let decoded_path: Result<UPath> = str_path.parse();
+        let decoded_path: Result<UHierPath> = str_path.parse();
         assert!(decoded_path.is_ok());
 
         assert_eq!(path, decoded_path.unwrap());
@@ -333,13 +317,13 @@ mod tests {
 
     #[test]
     fn test_path_validation() {
-        let valid_path: Result<UPath> = "a/b/c/d/e/f/g".parse();
+        let valid_path: Result<UHierPath> = "a/b/c/d/e/f/g".parse();
         assert!(valid_path.is_ok());
 
-        let invalid_path: Result<UPath> = "a/b/c//d/e/f/g".parse();
+        let invalid_path: Result<UHierPath> = "a/b/c//d/e/f/g".parse();
         assert!(invalid_path.is_err());
 
-        let invalid_path: Result<UPath> = "a//b/c//d/e/f///g".parse();
+        let invalid_path: Result<UHierPath> = "a//b/c//d/e/f///g".parse();
         assert!(invalid_path.is_err());
     }
 
@@ -375,7 +359,7 @@ mod tests {
         assert!(baz_result.is_ok());
         let baz_result = baz_result.unwrap();
 
-        let orphans = list_orphans(&open_result.pool.get().unwrap());
+        let orphans = list_roots(&open_result.pool.get().unwrap());
         assert!(orphans.is_ok());
         assert_eq!(orphans.unwrap().len(), 2);
 
