@@ -9,7 +9,7 @@ pub mod hierarchies;
 pub mod inner;
 pub mod lang;
 
-use crate::addressing::Address;
+use crate::addressing::{Address, Addressable};
 use crate::database::constants::{IS_OF_TYPE_ATTR, TYPE_ADDR, TYPE_HAS_ATTR, TYPE_INVARIANT};
 use crate::database::entry::{Entry, EntryValue};
 use crate::database::inner::models;
@@ -118,43 +118,35 @@ pub fn retrieve_object<C: Connection<Backend = Sqlite>>(
 ) -> Result<Vec<Entry>> {
     use crate::database::inner::schema::data::dsl::*;
 
-    let matches = data
+    let primary = data
         .filter(entity.eq(object_address.encode()?))
         .or_filter(value.eq(EntryValue::Address(object_address).to_string()?))
         .load::<models::Entry>(connection)?;
-    let entries = matches
+
+    let entries = primary
         .iter()
         .map(Entry::try_from)
-        .filter_map(Result::ok)
-        .collect();
+        .collect::<Result<Vec<Entry>>>()?;
 
-    Ok(entries)
-}
-
-pub fn bulk_retrieve_objects<C: Connection<Backend = Sqlite>>(
-    connection: &C,
-    object_addresses: Vec<Address>,
-) -> Result<Vec<Entry>> {
-    use crate::database::inner::schema::data::dsl::*;
-
-    let matches = data
+    let secondary = data
         .filter(
             entity.eq_any(
-                object_addresses
+                entries
                     .iter()
-                    .filter_map(|addr| addr.encode().ok()),
+                    .map(|e| e.address())
+                    .filter_map(Result::ok)
+                    .map(|addr| addr.encode())
+                    .collect::<Result<Vec<Vec<u8>>>>()?,
             ),
         )
-        // .or_filter(value.eq(EntryValue::Address(object_address).to_str()?))
         .load::<models::Entry>(connection)?;
 
-    let entries = matches
+    let secondary_entries = secondary
         .iter()
         .map(Entry::try_from)
-        .filter_map(Result::ok)
-        .collect();
+        .collect::<Result<Vec<Entry>>>()?;
 
-    Ok(entries)
+    Ok([entries, secondary_entries].concat())
 }
 
 pub fn remove_object<C: Connection<Backend = Sqlite>>(
