@@ -49,6 +49,11 @@ fn main() -> Result<()> {
                 .long("no-browser")
                 .help("Do not open web browser with the UI."),
         )
+        .arg(
+            Arg::with_name("NO_DESKTOP")
+                .long("no-desktop")
+                .help("Disable desktop features (webbrowser, native file opening)"),
+        )
         .arg(
             Arg::with_name("NO_UI")
                 .long("no-ui")
@@ -97,7 +102,19 @@ fn main() -> Result<()> {
     info!("Starting server at: {}", &bind);
 
     let state = routes::State {
-        vault_name: matches.value_of("VAULT_NAME").map(|s| s.to_string()),
+        vault_name: Some(
+            matches
+                .value_of("VAULT_NAME")
+                .map(|s| s.to_string())
+                .unwrap_or_else(|| {
+                    vault_path
+                        .iter()
+                        .last()
+                        .unwrap()
+                        .to_string_lossy()
+                        .into_owned()
+                }),
+        ),
         directory: vault_path.clone(),
         db_pool: db_pool.clone(),
         job_container: job_container.clone(),
@@ -120,6 +137,7 @@ fn main() -> Result<()> {
             .service(routes::delete_object)
             .service(routes::api_refresh)
             .service(routes::list_hier)
+            .service(routes::list_hier_roots)
             .service(routes::latest_files)
             .service(routes::get_file)
             .service(routes::get_jobs)
@@ -139,10 +157,13 @@ fn main() -> Result<()> {
         actix::spawn(filesystem::rescan_vault(db_pool, vault_path, job_container));
     }
 
-    if !matches.is_present("NO_BROWSER") && ui_enabled {
-        let ui_result = webbrowser::open(&format!("http://localhost:{}", bind.port()));
-        if ui_result.is_err() {
-            warn!("Could not open UI in browser!");
+    #[cfg(feature = "desktop")]
+    {
+        if !matches.is_present("NO_BROWSER") && ui_enabled {
+            let ui_result = webbrowser::open(&format!("http://localhost:{}", bind.port()));
+            if ui_result.is_err() {
+                warn!("Could not open UI in browser!");
+            }
         }
     }
 
