@@ -1,6 +1,6 @@
 use crate::addressing::{Address, Addressable};
 use crate::database::entry::{Entry, InEntry};
-use crate::database::hierarchies::{list_node, UPath};
+use crate::database::hierarchies::{list_roots, resolve_path, UHierPath};
 use crate::database::lang::Query;
 use crate::database::{
     get_latest_files, insert_entry, query, remove_object, retrieve_file, retrieve_object, DbPool,
@@ -9,10 +9,10 @@ use crate::util::hash::{decode, encode};
 use crate::util::jobs::JobContainer;
 use actix_files::NamedFile;
 use actix_web::error::{ErrorBadRequest, ErrorInternalServerError, ErrorNotFound};
-use actix_web::{delete, error, get, post, put, web, Error, HttpResponse};
+use actix_web::{delete, error, get, post, put, web, Either, Error, HttpResponse};
 use anyhow::Result;
 use futures_util::StreamExt;
-use log::{debug, trace};
+use log::{debug, info, trace};
 use serde::Deserialize;
 use serde_json::json;
 use std::collections::HashMap;
@@ -20,6 +20,9 @@ use std::convert::TryFrom;
 use std::path::PathBuf;
 use std::sync::{Arc, RwLock};
 
+#[cfg(feature = "desktop")]
+use is_executable::IsExecutable;
+
 const VERSION: &str = env!("CARGO_PKG_VERSION");
 
 #[derive(Clone)]
@@ -30,22 +33,44 @@ pub struct State {
     pub job_container: Arc<RwLock<JobContainer>>,
 }
 
+#[derive(Deserialize)]
+pub struct RawRequest {
+    native: Option<String>,
+}
+
 #[get("/api/raw/{hash}")]
-pub async fn get_raw(state: web::Data<State>, hash: web::Path<String>) -> Result<NamedFile, Error> {
+pub async fn get_raw(
+    state: web::Data<State>,
+    web::Query(query): web::Query<RawRequest>,
+    hash: web::Path<String>,
+) -> Result<Either<NamedFile, HttpResponse>, Error> {
     let address = Address::decode(&decode(hash.into_inner()).map_err(ErrorInternalServerError)?)
         .map_err(ErrorInternalServerError)?;
     if let Address::Hash(hash) = address {
         let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-        let response = retrieve_file(&connection, hash);
+        let files = retrieve_file(&connection, hash).map_err(ErrorInternalServerError)?;
+        if let Some(file) = files.get(0) {
+            let file_path = state.directory.join(&file.path);
 
-        debug!("{:?}", response);
-
-        match response {
-            Ok(result) => match result.get(0) {
-                Some(file) => Ok(NamedFile::open(state.directory.join(&file.path))?),
-                None => Err(error::ErrorNotFound("NOT FOUND")),
-            },
-            Err(e) => Err(error::ErrorInternalServerError(e)),
+            if !query.native.is_some() {
+                Ok(Either::A(NamedFile::open(file_path)?))
+            } else {
+                if cfg!(feature = "desktop") {
+                    #[cfg(feature = "desktop")]
+                    {
+                        info!("Opening {:?}...", file_path);
+                        if !file_path.is_executable() {
+                            opener::open(file_path).map_err(error::ErrorServiceUnavailable)?;
+                            return Ok(Either::B(HttpResponse::NoContent().finish()));
+                        }
+                    }
+                    Err(error::ErrorForbidden(""))
+                } else {
+                    Err(error::ErrorBadRequest("Desktop features not enabled."))
+                }
+            }
+        } else {
+            Err(error::ErrorNotFound("NOT FOUND"))
         }
     } else {
         Err(ErrorBadRequest("Address does not refer to a file."))
@@ -84,34 +109,36 @@ pub async fn get_query(
     Ok(HttpResponse::Ok().json(&result))
 }
 
+trait EntriesAsHash {
+    fn as_hash(&self) -> Result<HashMap<String, &Entry>>;
+}
+
+impl EntriesAsHash for Vec<Entry> {
+    fn as_hash(&self) -> Result<HashMap<String, &Entry>> {
+        let mut result: HashMap<String, &Entry> = HashMap::new();
+        for entry in self {
+            result.insert(encode(entry.address()?.encode()?), entry);
+        }
+
+        Ok(result)
+    }
+}
+
 #[get("/api/obj/{address_str}")]
 pub async fn get_object(
     state: web::Data<State>,
     address_str: web::Path<String>,
 ) -> Result<HttpResponse, Error> {
     let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-    let response: Result<Vec<Entry>> = retrieve_object(
+    let result: Vec<Entry> = retrieve_object(
         &connection,
         Address::decode(&decode(address_str.into_inner()).map_err(ErrorBadRequest)?)
             .map_err(ErrorBadRequest)?,
-    );
+    )
+    .map_err(ErrorInternalServerError)?;
 
-    debug!("{:?}", response);
-
-    let mut result: HashMap<String, Entry> = HashMap::new();
-    for entry in response.map_err(error::ErrorInternalServerError)? {
-        result.insert(
-            encode(
-                entry
-                    .address()
-                    .map_err(ErrorInternalServerError)?
-                    .encode()
-                    .map_err(ErrorInternalServerError)?,
-            ),
-            entry,
-        );
-    }
-    Ok(HttpResponse::Ok().json(result))
+    debug!("{:?}", result);
+    Ok(HttpResponse::Ok().json(result.as_hash().map_err(ErrorInternalServerError)?))
 }
 
 const MAX_SIZE: usize = 1_000_000;
@@ -172,13 +199,38 @@ pub async fn list_hier(
     path: web::Path<String>,
 ) -> Result<HttpResponse, Error> {
     let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
-    let upath: UPath = path.into_inner().parse().map_err(ErrorBadRequest)?;
-    trace!("Listing path \"{}\"", upath);
-    let entries: Vec<Entry> = list_node(&connection, &upath)
-        .await
-        .map_err(ErrorNotFound)?; // todo: 500 if actual error occurs
+    if path.is_empty() {
+        Ok(HttpResponse::MovedPermanently()
+            .header("Location", "/api/hier_roots")
+            .finish())
+    } else {
+        let upath: UHierPath = path.into_inner().parse().map_err(ErrorBadRequest)?;
+        trace!("Listing path \"{}\"", upath);
 
-    Ok(HttpResponse::Ok().json(entries))
+        // todo: 500 if actual error occurs
+        let path = resolve_path(&connection, &upath, false).map_err(ErrorNotFound)?;
+        match path.last() {
+            Some(addr) => Ok(HttpResponse::Found()
+                .header("Location", format!("/api/obj/{}", addr))
+                .finish()),
+            None => Ok(HttpResponse::NotFound().finish()),
+        }
+    }
+}
+
+#[get("/api/hier_roots")]
+pub async fn list_hier_roots(state: web::Data<State>) -> Result<HttpResponse, Error> {
+    let connection = state.db_pool.get().map_err(ErrorInternalServerError)?;
+
+    let result = list_roots(&connection)
+        .map_err(ErrorInternalServerError)?
+        .into_iter()
+        .map(|root| retrieve_object(&connection, root))
+        .collect::<Result<Vec<Vec<Entry>>>>()
+        .map_err(ErrorInternalServerError)?
+        .concat();
+
+    Ok(HttpResponse::Ok().json(result.as_hash().map_err(ErrorInternalServerError)?))
 }
 
 #[post("/api/refresh")]
