@@ -4,14 +4,14 @@ export type VALUE_TYPE = "Value" | "Address" | "Invalid";
 export interface IEntry {
   entity: Address;
   attribute: string;
-  value: { t: VALUE_TYPE; c: string };
+  value: { t: VALUE_TYPE; c: string | number };
 }
 
 export interface ListingResult {
   [key: string]: IEntry;
 }
 
-export type OrderedListing = [Address, IEntry][];
+// export type OrderedListing = [Address, IEntry][];
 
 export interface Job {
   title: string;
