@@ -1,12 +1,11 @@
-import type { SvelteComponent, SvelteComponentTyped } from "svelte";
-import List from "../components/widgets/List.svelte";
+import Table from "../components/widgets/Table.svelte";
 
 export class UpType {
   address: string;
   name: string | null = null;
   attributes: string[] = [];
 
-  constructor(address: string) {
+  constructor(address?: string) {
     this.address = address;
   }
 
@@ -19,6 +18,8 @@ export class UpType {
   }
 }
 
+export const UNTYPED = new UpType("UNTYPED");
+
 export interface Component {
   component: any; // TODO
   props?: { [key: string]: unknown };
@@ -41,9 +42,9 @@ const TYPE_WIDGETS: { [key: string]: Widget } = {
     icon: "folder",
     components: [
       {
-        component: List,
+        component: Table,
         props: {
-          attribute: "HAS",
+          columns: "value",
         },
       },
     ],
